"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

const console_proxy_1 = require("../console-proxy");

let instance;

class Logger {
  constructor(args = {}) {
    this._defaultConsoleOptions = {
      level: "info",
      levels: Logger._levels
    };
    this._defaultWinstonOptions = {
      exitOnError: false,
      level: "info",
      levels: Logger._levels,
      transports: {
        Console: {
          colorize: true,
          handleExceptions: true,
          json: true,
          timestamp: () => new Date().toISOString()
        }
      }
    };
    const consoleOptions = args.consoleOptions,
          _args$newInstance = args.newInstance,
          newInstance = _args$newInstance === void 0 ? false : _args$newInstance,
          winstonOptions = args.winstonOptions;
    if (instance && !newInstance) return instance;

    if (process.env.WEB_ENV) {
      const ConsolePrxy = require("../console-proxy").default;

      this._client = this._initializeConsole(ConsolePrxy, consoleOptions);
    } else {
      const winston = require("winston");

      this._client = this._initializeWinston(winston, winstonOptions);
    }

    instance = this;
    return instance;
  }

  get client() {
    return this._client;
  }

  debug(message, meta) {
    this._log("debug", message, meta);
  }

  error(message, meta) {
    this._log("error", message, meta);
  }

  info(message, meta) {
    this._log("info", message, meta);
  }

  verbose(message, meta) {
    this._log("verbose", message, meta);
  }

  warn(message, meta) {
    this._log("warn", message, meta);
  }

  _initializeConsole(ConsolePrxy, options = {}) {
    return new console_proxy_1.default(Object.assign({}, this._defaultConsoleOptions, options));
  }

  _initializeWinston(winston, options = {}) {
    const mergedOptions = this._mergeWinstonOptions(winston, options);

    const transports = [];

    if (mergedOptions.transports) {
      const mergedOptionsTransports = mergedOptions.transports;
      Object.keys(mergedOptionsTransports).map(key => {
        const WinstonTransport = winston.transports[key];
        transports.push(new WinstonTransport(mergedOptionsTransports[key]));
      });
    }

    return new winston.Logger(Object.assign({}, mergedOptions, {
      transports
    }));
  }

  _log(level, message, meta) {
    if (process.env.ISO_LOG === "false") return;

    this._client.log(level, message, meta);
  }

  _mergeWinstonOptions(winston, options) {
    if (options.transports) {
      const transportOptions = options.transports;
      const validOptions = {};
      Object.keys(transportOptions).forEach(key => {
        if (key in winston.transports) {
          validOptions[key] = transportOptions[key];
        }
      });
      options.transports = validOptions;
    }

    return Object.assign({}, this._defaultWinstonOptions, options);
  }

}

Logger._levels = {
  error: 0,
  warn: 1,
  info: 2,
  verbose: 3,
  debug: 4
};
exports.Logger = Logger;
//# sourceMappingURL=index.js.map
